@@ -25,6 +25,7 @@
 #include "gtk-utils.h"
 #include "kazehakase.h"
 #include "kz-icons.h"
+#include "kz-navi.h"
 
 #define KZ_ACTIONS_DYNAMIC_COPY_IN_USER_FORMAT_KEY "KzActionsDynamic::CopyInUserFormat"
 
@@ -205,3 +206,56 @@ kz_actions_dynamic_append_copy_in_user_format_menuitem (KzTabLabel *kztab,
 		g_free(title);
 	}
 }
+
+static void
+cb_add_feed_bookmark_menuitem_activate (GtkWidget *widget, KzNavi *navi)
+{
+	kz_navi_add_bookmark(navi);
+}
+
+void
+kz_actions_dynamic_append_add_feed_bookmark (KzEmbed *kzembed,
+					     GtkMenuItem *menu)
+{
+	GtkWidget *add_feed_bookmark_submenu = NULL;
+	GtkWidget *menuitem;
+	GList *nav_link, *nav_links;
+	KzNavi *navi;
+	gchar *title;
+
+	add_feed_bookmark_submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menu));
+	if (add_feed_bookmark_submenu)
+	{
+		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu));
+	}
+
+	nav_links = kz_embed_get_nav_links(kzembed, KZ_EMBED_LINK_RSS);
+
+	if (!nav_links) return;
+	
+	add_feed_bookmark_submenu = gtk_menu_new();
+	gtk_widget_show(add_feed_bookmark_submenu);
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu),
+				  add_feed_bookmark_submenu);
+
+	/* append available feed menuitem */
+	for (nav_link = nav_links;
+	     nav_link;
+	     nav_link = g_list_next(nav_link))
+	{
+		navi = nav_link->data;
+		if (!navi) continue;
+		
+		title = navi->title ? navi->title : _("Feed");
+
+		menuitem = gtk_menu_item_new_with_label(title);
+
+		g_signal_connect(menuitem, "activate",
+				 G_CALLBACK(cb_add_feed_bookmark_menuitem_activate),
+				 navi);
+
+		gtk_menu_shell_append(GTK_MENU_SHELL(add_feed_bookmark_submenu),
+				      menuitem);
+		gtk_widget_show(menuitem);
+	}
+}
