@@ -320,6 +320,12 @@ GtkActionEntry kz_actions_tab[] =
 	{"TabStop",  GTK_STOCK_STOP, N_("_Stop"),
 	 NULL, NULL, G_CALLBACK(act_tab_stop)},
 
+	{"TabAddFeedBookmark", NULL, N_("Add _feed to bookmarks"),
+	 NULL,  NULL, NULL},
+	
+	{"TabStop",  GTK_STOCK_STOP, N_("_Stop"),
+	 NULL, NULL, G_CALLBACK(act_tab_stop)},
+
 	{"TabClose", GTK_STOCK_CLOSE, N_("_Close"),
 	 NULL, NULL, G_CALLBACK(act_tab_close)},
 
@@ -411,7 +417,7 @@ set_popup_menu_sensitive (KzTabLabel *kztab)
 	KzWindow *kz;
 	GtkAction *action;
 	gint pos, num;
-	gboolean lock, auto_refresh, javascript, images;
+	gboolean lock, auto_refresh, javascript, images, has_feed;
 
 	g_return_if_fail(kztab);
 
@@ -427,6 +433,15 @@ set_popup_menu_sensitive (KzTabLabel *kztab)
 		     kz_embed_is_loading(kztab->kzembed),
 		     NULL);
 
+	action = gtk_action_group_get_action(kz->tabpop_actions,
+					     "TabAddFeedBookmark");
+	if (kztab->kzembed &&
+	    kz_embed_get_nav_link(kztab->kzembed, KZ_EMBED_LINK_RSS))
+		has_feed = TRUE;
+	else
+		has_feed = FALSE;
+	g_object_set(action, "sensitive", has_feed, NULL);
+	
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabCloseAllBackward");
 	g_object_set(action, "sensitive", !(pos == 0), NULL);
@@ -492,6 +507,15 @@ kz_actions_tab_popup_menu_modal (KzTabLabel *kztab, guint button, guint time)
 			(kztab, GTK_MENU_ITEM(extra_menu));
 	}
 
+	/* add available feeds */
+	extra_menu = gtk_ui_manager_get_widget(kztab->kz->menu_merge,
+					       "/TabPopup/TabAddFeedBookmark");
+	if(extra_menu)
+	{
+		kz_actions_dynamic_append_add_feed_bookmark
+			(kztab->kzembed, GTK_MENU_ITEM(extra_menu));
+	}
+
 	set_popup_menu_sensitive(kztab);
 
 	g_signal_connect(popup_menu, "hide",
