@@ -49,6 +49,7 @@
 #include "kz-embed.h"
 #include "kz-actions-download.h"
 #include "kz-actions-popup.h"
+#include "kz-actions-dynamic.h"
 #include "kz-about-dialog.h"
 #include "kz-paned.h"
 #include "kz-bookmark-menu.h"
@@ -56,6 +57,7 @@
 #include "kz-popup-tablist.h"
 #include "eggregex.h"
 #include "kz-proxy-folder.h"
+#include "kz-icons.h"
 
 #include <time.h>
 
@@ -621,18 +623,47 @@ act_find (GtkAction *action, KzWindow *kz)
 static void
 act_prev_tab (GtkAction *action, KzWindow *kz)
 {
-	g_return_if_fail(KZ_IS_WINDOW(kz));
+	GtkNotebook *notebook;
+	gboolean circulation = FALSE;
 
-	gtk_notebook_prev_page(GTK_NOTEBOOK(kz->notebook));
+	notebook = GTK_NOTEBOOK(kz->notebook);
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+	KZ_CONF_GET("Tab", "wheel_circulation", circulation, BOOL);
+
+	if (circulation &&
+	    gtk_notebook_get_current_page(notebook) == 0)
+	{
+		gtk_notebook_set_current_page
+			(notebook, gtk_notebook_get_n_pages(notebook) - 1);
+	}
+	else
+	{
+		gtk_notebook_prev_page(notebook);
+	}
 }
 
 
 static void
 act_next_tab (GtkAction *action, KzWindow *kz)
 {
+	GtkNotebook *notebook;
+	gboolean circulation = FALSE;
+
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	gtk_notebook_next_page(GTK_NOTEBOOK(kz->notebook));
+	notebook = GTK_NOTEBOOK(kz->notebook);
+	KZ_CONF_GET("Tab", "wheel_circulation", circulation, BOOL);
+	
+	if (circulation && 
+	    gtk_notebook_get_current_page(notebook) == 
+	    gtk_notebook_get_n_pages(notebook) - 1)
+	{
+		gtk_notebook_set_current_page(notebook, 0);
+	}
+	else
+	{
+		gtk_notebook_next_page(notebook);
+	}
 }
 
 
@@ -866,108 +897,6 @@ act_copy_url (GtkAction *action, KzWindow *kz)
 	gtkutil_copy_text(text);
 }
 
-static void
-act_copy_in_user_format(GtkAction *action, KzWindow *kz)
-{
-	GtkWidget *widget;
-	gchar *text;
-	gchar *format;
-	gchar **splited = NULL;
-	gint counts = 0;
-	gint i;
-	gchar *str = NULL;
-
-	g_return_if_fail(KZ_IS_WINDOW (kz));
-	
-	widget = KZ_WINDOW_CURRENT_PAGE(kz);
-	 if (!KZ_IS_EMBED(widget)) return;
-
-	text = kz_embed_get_selection_string(KZ_EMBED(widget));
-	if (!text) text = g_strdup("");
-
-	format = KZ_CONF_GET_STR("Global" , "copy_document_format1");
-	if(!format) return;
-
-	for(i = 0;format[i] != '\0';i++)
-		if(format[i] == '%') counts++;
-	counts++;
-	
-	if(counts != 1)
-	{
-		gchar **temp = g_alloca(sizeof(gchar*) * (counts + 1));
-		splited = g_strsplit(format , "%" , counts);
-		gint st = 0;
-
-		for(i = 0; i < counts-1; i++)
-		{
-			if(st == 0)
-			{
-				/* first state */
-				temp[i] = splited[i];
-				st = 1;
-			}
-			else if (st == 2)
-			{
-				/* after %t% and so on */
-				temp[i] = splited[i];
-				st = 1;
-			}
-			else if (splited[i][0] == '\0')
-			{
-				/* "%%" */
-				temp[i] = "%";
-				st = 0;
-			}
-			else if (splited[i][1] != '\0')
-			{
-				/* %text% */
-				temp[i] = splited[i];
-				st = 1;
-			}
-			else if(splited[i][0] == 't')
-			{
-				/* %t% */
-				temp[i] = (gchar*)kz_embed_get_title(KZ_EMBED(widget));
-				st = 2;
-			}
-			else if(splited[i][0] == 'u')
-			{
-				/* %u% */
-				temp[i] = (gchar*)kz_embed_get_location(KZ_EMBED(widget));
-				st = 2;
-			}
-			else if(splited[i][0] == 's')
-			{
-				/* %s% */
-				temp[i] = text;
-				st = 2;
-			}
-			else
-			{
-				/* '%' + other char + '%' */
-				temp[i] = splited[i];
-				st = 1;
-			}
-		}
-		temp[counts-1] = splited[counts-1];
-		temp[counts] = NULL;
-
-		str = g_strjoinv(NULL, temp);
-		g_strfreev(splited);
-	}
-	else
-	{
-		str = g_strdup(format);
-	}
-
-	g_free(format);
-
-	gtkutil_copy_text(str);
-
-	if(str) g_free(str);
-	if(text) g_free(text);
-}
-
 static void
 act_get_body_text (GtkAction *action, KzWindow *kz)
 {
@@ -1217,6 +1146,20 @@ act_add_bookmark (GtkAction *action, KzWindow *kz)
 	g_object_unref(G_OBJECT(bookmark));
 }
 
+static void
+act_add_first_feed_bookmark (GtkAction *action, KzWindow *kz)
+{
+	KzEmbed *embed;
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	g_return_if_fail(KZ_IS_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	embed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
+	
+	kz_navi_add_bookmark(kz_embed_get_nth_nav_link(embed,
+						       KZ_EMBED_LINK_RSS,
+						       0));
+}
 
 static void
 act_remove_bookmark (GtkAction *action, KzWindow *kz)
@@ -1568,6 +1511,18 @@ act_about (GtkAction *action, KzWindow *kz)
 	kz_about_dialog_new(kz);
 }
 
+static void
+act_open_kazehakase_website(GtkAction *action, KzWindow *kz)
+{
+	GtkWidget *widget;
+	gint num;
+
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	widget = kz_window_open_new_tab(kz, KAZEHAKASE_URI);
+	num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook), widget);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), num);
+}
 
 static void
 act_copy_tab (GtkAction *action, KzWindow *kz)
@@ -2043,6 +1998,8 @@ GtkActionEntry kz_actions[] =
   {"OpenAllBookmarks",          NULL, N_("Open a_ll bookmarks"),             NULL,  NULL, G_CALLBACK(act_open_all_bookmarks)},
   {"OpenAllBookmarksRecursive", NULL, N_("Open all bookmarks _recursively"), NULL,  NULL, G_CALLBACK(act_open_all_bookmarks_recursive)},
   {"AddBookmark",               NULL, N_("_Add to bookmarks"),               NULL,  NULL, G_CALLBACK(act_add_bookmark)},
+  {"AddFirstFeedBookmark",      NULL, N_("Add _1st feed to bookmarks"),      NULL,  NULL, G_CALLBACK(act_add_first_feed_bookmark)},
+  {"AddFeedBookmark",           NULL, N_("Add _feed to bookmarks"),          NULL,  NULL, NULL},
   {"RemoveBookmark",            NULL, N_("_Remove the bookmark"),            NULL,  NULL, G_CALLBACK(act_remove_bookmark)},
   {"EditBookmarks",             NULL, N_("_Edit bookmarks..."),              NULL,  NULL, G_CALLBACK(act_edit_bookmark)},
   {"EditBookmarkBars",          NULL, N_("Edit bookmark _bars..."),          NULL,  NULL, G_CALLBACK(act_edit_bookmark_bars)},
@@ -2062,6 +2019,7 @@ GtkActionEntry kz_actions[] =
   {"ShowPassword", NULL, N_("_Show Password Manager"), NULL,  NULL, G_CALLBACK(act_show_password_manager)},
   {"ShowCopyFormat", NULL, N_("_Show Copy Format"), NULL,  NULL, G_CALLBACK(act_show_copy_format)},
   {"About",        NULL, N_("About Kazehakase"),       NULL,  N_("Display credits"), G_CALLBACK(act_about)},
+  {"OpenKazehakaseWebsite",        NULL, N_("Open Kazehakase Website"),       NULL,  N_("Open Kazehakase website in new tab"), G_CALLBACK(act_open_kazehakase_website)},
   
   {"PopupContextMenu", NULL, N_("Popup context menu"), SHFT"F10",  N_("Popup context menu"), G_CALLBACK(act_popup_context_menu)},
 };
@@ -2108,9 +2066,10 @@ static void
 set_popup_menu_sensitive (KzWindow *kz, KzBookmark *bookmark)
 {
 	KzBookmark *bookmark_file;
+	KzEmbed *kzembed;
 	GtkAction *action;
 	gboolean has_link, is_folder, is_file, has_children;
-	gboolean children_is_editable, is_editable;
+	gboolean children_is_editable, is_editable, has_feed;
 
 	g_return_if_fail(kz && bookmark);
 
@@ -2127,6 +2086,11 @@ set_popup_menu_sensitive (KzWindow *kz, KzBookmark *bookmark)
 		children_is_editable = TRUE;
 	else
 		children_is_editable = FALSE;
+	kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
+	if (kzembed && kz_embed_get_nav_link(kzembed, KZ_EMBED_LINK_RSS))
+		has_feed = TRUE;
+	else
+		has_feed = FALSE;
 
 	action = gtk_action_group_get_action(kz->actions,
 					     "OpenBookmarkInNewTab");
@@ -2141,6 +2105,10 @@ set_popup_menu_sensitive (KzWindow *kz, KzBookmark *bookmark)
 	g_object_set(action, "sensitive", ((is_file && children_is_editable) ||
 					  (!is_file && is_editable)), NULL);
 
+	action = gtk_action_group_get_action(kz->actions,
+					     "AddFeedBookmark");
+	g_object_set(action, "sensitive", has_feed, NULL);
+
 	action = gtk_action_group_get_action(kz->actions,
 					     "RemoveBookmark");
 	g_object_set(action, "sensitive", is_editable, NULL);
@@ -2279,10 +2247,10 @@ kz_actions_set_tab_sensitive (KzWindow *kz, KzEmbed *kzembed)
 	{
 		action = gtk_action_group_get_action(kz->actions,
 						     "PrevTab");
+		KZ_WINDOW_SET_SENSITIVE(kz, "PrevTab", TRUE);
 		if (pos == 0)
 		{
 			combine_favicon(action, NULL, kz);
-			KZ_WINDOW_SET_SENSITIVE(kz, "PrevTab", FALSE);
 			KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllBackwardTab", FALSE);
 		}
 		else
@@ -2291,16 +2259,15 @@ kz_actions_set_tab_sensitive (KzWindow *kz, KzEmbed *kzembed)
 			const gchar* uri = kz_embed_get_location(prev);
 			/* set favicon */
 			combine_favicon(action, uri, kz);
-			KZ_WINDOW_SET_SENSITIVE(kz, "PrevTab", TRUE);
 			KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllBackwardTab", TRUE);
 		}
 
 		action = gtk_action_group_get_action(kz->actions,
 						     "NextTab");
+		KZ_WINDOW_SET_SENSITIVE(kz, "NextTab", TRUE);
 		if (pos == num - 1)
 		{
 			combine_favicon(action,	NULL, kz);
-			KZ_WINDOW_SET_SENSITIVE(kz, "NextTab", FALSE);
 			KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllForwardTab", FALSE);
 		}
 		else
@@ -2309,7 +2276,6 @@ kz_actions_set_tab_sensitive (KzWindow *kz, KzEmbed *kzembed)
 			const gchar* uri = kz_embed_get_location(next);
 			/* set favicon */
 			combine_favicon(action,	uri, kz);
-			KZ_WINDOW_SET_SENSITIVE(kz, "NextTab", TRUE);
 			KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllForwardTab", TRUE);
 		}
 	}
@@ -2319,6 +2285,15 @@ kz_actions_set_tab_sensitive (KzWindow *kz, KzEmbed *kzembed)
 		KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllForwardTab",       FALSE);
 		KZ_WINDOW_SET_SENSITIVE(kz, "CloseAllTabWithoutActive", FALSE);
 	}
+
+	if (kzembed && kz_embed_get_nav_link(kzembed, KZ_EMBED_LINK_RSS))
+	{
+		KZ_WINDOW_SET_SENSITIVE(kz, "AddFirstFeedBookmark", TRUE);
+	}
+	else
+	{
+		KZ_WINDOW_SET_SENSITIVE(kz, "AddFirstFeedBookmark", FALSE);
+	}
 }
 
 void
@@ -2373,6 +2348,7 @@ kz_actions_set_sensitive (KzWindow *kz, KzEmbed *kzembed)
 			if (can_stop_all) break;
 		}
 	}
+	
 
 	if (num > 0)
 	{
@@ -2485,6 +2461,15 @@ kz_actions_set_sensitive (KzWindow *kz, KzEmbed *kzembed)
 		gtk_action_group_set_sensitive (kz->tabpop_actions, FALSE);
 	}
 
+	if (kzembed && kz_embed_get_nav_link(kzembed, KZ_EMBED_LINK_RSS))
+	{
+		KZ_WINDOW_SET_SENSITIVE(kz, "AddFirstFeedBookmark", TRUE);
+	}
+	else
+	{
+		KZ_WINDOW_SET_SENSITIVE(kz, "AddFirstFeedBookmark", FALSE);
+	}
+	
 	kz_actions_set_selection_sensitive (kz, kzembed);
 	
 	/* Sidebar position menu */
@@ -2625,6 +2610,7 @@ kz_actions_popup_bookmark_menu_modal (KzWindow *kz, KzBookmark *bookmark,
 				      guint button, guint time)
 {
 	GtkWidget *popup_menu = NULL;
+	GtkWidget *extra_menu = NULL;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
@@ -2634,6 +2620,17 @@ kz_actions_popup_bookmark_menu_modal (KzWindow *kz, KzBookmark *bookmark,
 					       "/BookmarkPopup");
 	if (!popup_menu) return;
 
+	extra_menu = gtk_ui_manager_get_widget(kz->menu_merge,
+					       "/BookmarkPopup/AddFeedBookmark");
+	if(extra_menu)
+	{
+		GtkWidget *kzembed = KZ_WINDOW_CURRENT_PAGE(kz);
+		if (kzembed)
+			kz_actions_dynamic_append_add_feed_bookmark
+				(KZ_EMBED(kzembed),
+				 GTK_MENU_ITEM(extra_menu));
+	}
+	
 	g_signal_connect(popup_menu, "hide",
 			 G_CALLBACK(cb_popup_menu_hide), NULL);
 	set_popup_menu_sensitive (kz, bookmark);
