@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu.c,v 1.8 2005/07/20 22:43:23 ikezoe Exp $
+ *  $Id: kz-bookmark-menu.c,v 1.9 2005/08/17 10:38:05 ikezoe Exp $
  */
 
 #include "kz-bookmark-menu.h"
@@ -26,11 +26,7 @@
 #include <glib/gi18n.h>
 
 #include "kazehakase.h"
-#include "kz-bookmark-editor.h"
-#include "kz-actions.h"
-#include "kz-favicon.h"
 #include "utils.h"
-#include "kz-icons.h"
 #include "kz-bookmark-menu-item-list.h"
 #include "kz-bookmark-menu-item.h"
 #include "kz-bookmark-folder-menu-item.h"
