@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.h,v 1.32 2004/12/05 08:06:06 tyuyu Exp $
+ *  $Id: kz-bookmark.h,v 1.33 2005/07/08 09:06:22 ikezoe Exp $
  */
 
 #ifndef __KZ_BOOKMARK_H__
@@ -80,7 +80,7 @@ struct _KzBookmarkClass
 
 /* GType */
 GType		kz_bookmark_type_get_type	(void) G_GNUC_CONST;
-GType		kz_bookmark_get_type		(void);
+GType		kz_bookmark_get_type		(void) G_GNUC_CONST; 
 
 /* boomark item creating functions */
 KzBookmark     *kz_bookmark_new			(void);
