@@ -17,16 +17,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-view.c,v 1.26 2005/05/09 01:20:12 ikezoe Exp $
+ *  $Id: kz-bookmarks-view.c,v 1.28 2005/07/21 05:02:01 ikezoe Exp $
  */
 
 #include "kz-bookmarks-view.h"
 #include <glib/gi18n.h>
+#include <string.h>
 
 #include "gobject-utils.h"
 #include "kz-bookmark-file.h"
 #include "kz-smart-bookmark.h"
 #include "kz-favicon.h"
+#include "kz-icons.h"
 
 
 enum {
