@@ -36,7 +36,6 @@ G_BEGIN_DECLS
 
 typedef struct _KzSmartBookmark      KzSmartBookmark;
 typedef struct _KzSmartBookmarkClass KzSmartBookmarkClass;
-typedef struct _KzSmartBookmarkPriv  KzSmartBookmarkPriv;
 
 typedef struct _KzSmartBookmarkProperty
 {
@@ -49,8 +48,6 @@ typedef struct _KzSmartBookmarkProperty
 struct _KzSmartBookmark
 {
 	KzBookmark parent;
-
-	KzSmartBookmarkPriv *priv;
 };
 
 struct _KzSmartBookmarkClass
@@ -74,6 +71,9 @@ gchar	    *kz_smart_bookmark_get_smart_uri   (KzSmartBookmark *bookmark,
 const GList *kz_smart_bookmark_get_smart_list  (KzSmartBookmark *bookmark);
 void         kz_smart_bookmark_set_smart_list  (KzSmartBookmark *bookmark,
 					        GList *list);
+KzBookmark  *kz_smart_bookmark_get_history     (KzSmartBookmark *bookmark);
+void         kz_smart_bookmark_set_history     (KzSmartBookmark *bookmark,
+						KzBookmark  *history);
 
 G_END_DECLS
 
