@@ -36,8 +36,8 @@ G_BEGIN_DECLS
 
 #define KAZEHAKASE_URI "http://kazehakase.sourceforge.jp/"
 
-#define HISTORY_INDEX "/.kazehakase/history_index "
-#define HISTORY_DIR   "/.kazehakase/history/"
+#define HISTORY_INDEX "/."PACKAGE"/history_index "
+#define HISTORY_DIR   "/."PACKAGE"/history/"
 
 
 typedef enum {
