@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-khtml.c,v 1.18 2005/05/09 01:20:12 ikezoe Exp $
+ *  $Id: kz-khtml.c,v 1.19 2005/08/13 09:33:14 kous Exp $
  */
 
 #include <glib/gi18n.h>
@@ -157,7 +157,11 @@ kz_khtml_iface_init (KzEmbedIFace *iface)
 /*	iface->can_go_up              = NULL; */
 	iface->can_go_nav_link        = NULL;
 	iface->go_nav_link            = NULL;
+	iface->append_nav_link        = NULL;
 	iface->set_nav_link           = NULL;
+	iface->set_nth_nav_link       = NULL;
+	iface->get_nav_link           = NULL;
+	iface->get_nth_nav_link       = NULL;
 	iface->go_history_index       = NULL;
 	iface->do_command             = NULL;
 	iface->can_do_command         = NULL;
