@@ -68,7 +68,7 @@ struct _KzDownloaderIcon
 	gint fraction;
 };
 
-GType      kz_download_box_get_type (void);
+GType      kz_download_box_get_type (void) G_GNUC_CONST;
 GtkWidget *kz_download_box_new      (KzWindow *kz);
 
 G_END_DECLS
