@@ -651,14 +651,70 @@ kz_embed_go_nav_link (KzEmbed *kzembed, KzEmbedNavLink link)
 
 
 void
-kz_embed_set_nav_link (KzEmbed *kzembed,
-		       const gchar *name,
-		       const gchar *link)
+kz_embed_append_nav_link (KzEmbed *kzembed,
+			  KzEmbedNavLink link,
+			  KzNavi *navi)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_nav_link);
+	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->append_nav_link);
 
-	KZ_EMBED_GET_IFACE(kzembed)->set_nav_link(kzembed, name, link);
+	KZ_EMBED_GET_IFACE(kzembed)->append_nav_link(kzembed, link, navi);
+}
+
+void
+kz_embed_set_nav_link (KzEmbed *kzembed,
+		       KzEmbedNavLink link,
+		       KzNavi *navi)
+{
+	g_return_if_fail(KZ_IS_EMBED(kzembed));
+
+	if (KZ_EMBED_GET_IFACE(kzembed)->set_nav_link)
+		KZ_EMBED_GET_IFACE(kzembed)->set_nav_link(kzembed, link, navi);
+	else
+		kz_embed_set_nth_nav_link(kzembed, link, navi, 0);
+}
+
+void
+kz_embed_set_nth_nav_link (KzEmbed *kzembed,
+			   KzEmbedNavLink link,
+			   KzNavi *navi,
+			   guint n)
+{
+	g_return_if_fail(KZ_IS_EMBED(kzembed));
+	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_nth_nav_link);
+
+	KZ_EMBED_GET_IFACE(kzembed)->set_nth_nav_link(kzembed, link, navi, n);
+}
+
+
+KzNavi *
+kz_embed_get_nav_link (KzEmbed *kzembed, KzEmbedNavLink link)
+{
+	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
+
+	if (KZ_EMBED_GET_IFACE(kzembed)->get_nav_link)
+		return KZ_EMBED_GET_IFACE(kzembed)->get_nav_link(kzembed, link);
+	else
+		return kz_embed_get_nth_nav_link(kzembed, link, 0);
+}
+
+KzNavi *
+kz_embed_get_nth_nav_link (KzEmbed *kzembed, KzEmbedNavLink link, guint n)
+{
+	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
+	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_nth_nav_link, NULL);
+
+	return KZ_EMBED_GET_IFACE(kzembed)->get_nth_nav_link(kzembed, link, n);
+}
+
+
+GList *
+kz_embed_get_nav_links (KzEmbed *kzembed, KzEmbedNavLink link)
+{
+	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
+	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_nav_links, NULL);
+
+	return KZ_EMBED_GET_IFACE(kzembed)->get_nav_links(kzembed, link);
 }
 
 
