@@ -22,6 +22,7 @@
 #include <gtk/gtk.h>
 #include "kazehakase.h"
 #include "kz-embed-event.h"
+#include "kz-navi.h"
 
 #ifndef __KZ_EMBED_H__
 #define __KZ_EMBED_H__
@@ -67,7 +68,8 @@ typedef enum {
 	KZ_EMBED_LINK_INDEX,
 	KZ_EMBED_LINK_CONTENTS,
 	KZ_EMBED_LINK_START,
-	KZ_EMBED_LINK_RSS
+	KZ_EMBED_LINK_RSS,
+	KZ_EMBED_LINK_GUARD
 } KzEmbedNavLink;
 
 struct _KzEmbedIFace {
@@ -144,9 +146,23 @@ struct _KzEmbedIFace {
 						 KzEmbedNavLink	link);
 	void		(*go_nav_link)		(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
-	void		(*set_nav_link)		(KzEmbed       *kzembed,
-						 const gchar   *name,
-						 const gchar   *link);
+	void		(*append_nav_link)	(KzEmbed       *kzembed,
+						 KzEmbedNavLink link,
+						 KzNavi        *navi);
+	void		(*set_nav_link)	        (KzEmbed       *kzembed,
+						 KzEmbedNavLink link,
+						 KzNavi        *navi);
+	void		(*set_nth_nav_link)	(KzEmbed       *kzembed,
+						 KzEmbedNavLink link,
+						 KzNavi        *navi,
+						 guint          n);
+	KzNavi         *(*get_nav_link)		(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link);
+	KzNavi         *(*get_nth_nav_link)	(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link,
+						 guint          n);
+	GList          *(*get_nav_links)	(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link);
 	void		(*go_history_index)	(KzEmbed       *kzembed,
 						 gint		index);
 
@@ -340,9 +356,23 @@ gboolean	kz_embed_can_go_nav_link	(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
 void		kz_embed_go_nav_link		(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
+void		kz_embed_append_nav_link	(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link,
+						 KzNavi        *navi);
 void		kz_embed_set_nav_link		(KzEmbed       *kzembed,
-						 const gchar   *name,
-						 const gchar   *link);
+						 KzEmbedNavLink	link,
+						 KzNavi        *navi);
+void		kz_embed_set_nth_nav_link	(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link,
+						 KzNavi        *navi,
+						 guint          n);
+KzNavi         *kz_embed_get_nav_link		(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link);
+KzNavi         *kz_embed_get_nth_nav_link	(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link,
+						 guint          n);
+GList          *kz_embed_get_nav_links		(KzEmbed       *kzembed,
+						 KzEmbedNavLink	link);
 void		kz_embed_go_history_index	(KzEmbed       *kzembed,
 						 gint		index);
 
