@@ -30,7 +30,6 @@
 #include "kz-actions-tab.h"
 #include "kz-icons.h"
 #include "kz-favicon.h"
-#include "kz-input-event-box.h"
 #include "kz-bookmark-file.h"
 
 enum {
@@ -40,7 +39,8 @@ enum {
 };
 
 
-struct _KzTabLabelPriv
+typedef struct _KzTabLabelPrivate       KzTabLabelPrivate;
+struct _KzTabLabelPrivate
 {
 	gint width;
 	gint start_x, start_y;
@@ -50,6 +50,7 @@ struct _KzTabLabelPriv
 	guint auto_refresh_id;
 	gulong favicon_signal;
 };
+#define KZ_TAB_LABEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_TAB_LABEL, KzTabLabelPrivate))
 
 static gchar *label_color[KZ_TAB_LABEL_N_STATE];
 
@@ -62,7 +63,6 @@ static GObject *kz_tab_label_constructor        (GType type,
 						 guint n_props,
 						 GObjectConstructParam *props);
 static void     kz_tab_label_dispose            (GObject *object);
-static void     kz_tab_label_finalize           (GObject *object);
 static void     kz_tab_label_set_property       (GObject *object,
 						 guint prop_id,
 						 const GValue *value,
@@ -150,7 +150,6 @@ static GdkColor red = { 0, 0xffff, 0x0000, 0x0000 };
 KZ_OBJECT_GET_TYPE(kz_tab_label, "KzTabLabel", KzTabLabel,
 		   kz_tab_label_class_init, kz_tab_label_init,
 		   GTK_TYPE_HBOX)
-KZ_OBJECT_FINALIZE (kz_tab_label, KzTabLabel)
 
 static void
 kz_tab_label_class_init (KzTabLabelClass *klass)
@@ -166,7 +165,6 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 	/* GtkObject signals */
 	gobject_class->constructor  = kz_tab_label_constructor;
 	gobject_class->dispose      = kz_tab_label_dispose;
-	gobject_class->finalize     = kz_tab_label_finalize;
 	gobject_class->set_property = kz_tab_label_set_property;
 	gobject_class->get_property = kz_tab_label_get_property;
 
@@ -202,6 +200,7 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 				      KZ_TYPE_EMBED,
 				      G_PARAM_READWRITE |
 				      G_PARAM_CONSTRUCT_ONLY));
+	g_type_class_add_private (gobject_class, sizeof(KzTabLabelPrivate));
 }
 
 static void
@@ -211,27 +210,28 @@ kz_tab_label_init (KzTabLabel *kztab)
 	GtkRcStyle *style;
 	GtkRequisition size;
 	gint width, height;
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
 	kztab->kz           = NULL;
 	kztab->kzembed      = NULL;
 	kztab->state        = KZ_TAB_LABEL_STATE_NORMAL;
 	kztab->favicon      = gtk_image_new();
-	kztab->eventbox     = kz_input_event_box_new();
+	kztab->eventbox     = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(kztab->eventbox), FALSE);
 	kztab->label        = gtk_label_new(NULL);
 	kztab->close_button = gtk_button_new();
 	kztab->lock_button  = gtk_image_new_from_stock(KZ_STOCK_ANCHOR,
 						       GTK_ICON_SIZE_MENU);
 	kztab->tooltips     = gtk_tooltips_new();
 
-	kztab->priv = g_new0(KzTabLabelPriv, 1);
-	kztab->priv->width             = 80;
-	kztab->priv->start_x           = 0;
-	kztab->priv->start_y           = 0;
-	kztab->priv->moved             = FALSE;
-	kztab->priv->lock              = FALSE;
-	kztab->priv->auto_refresh      = FALSE;
-	kztab->priv->auto_refresh_id   = 0;
-	kztab->priv->favicon_signal    = 0;
+	priv->width             = 80;
+	priv->start_x           = 0;
+	priv->start_y           = 0;
+	priv->moved             = FALSE;
+	priv->lock              = FALSE;
+	priv->auto_refresh      = FALSE;
+	priv->auto_refresh_id   = 0;
+	priv->favicon_signal    = 0;
 	
 	kztab->favicon_pixbuf   = NULL;
 	kztab->circle           = NULL;
@@ -355,6 +355,7 @@ kz_tab_label_dispose (GObject *object)
 	KzTabLabel *kztab;
 
 	kztab = KZ_TAB_LABEL(object);
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
 	g_signal_handlers_disconnect_by_func(kz_global_profile,
 					     G_CALLBACK(cb_profile_changed),
@@ -378,11 +379,11 @@ kz_tab_label_dispose (GObject *object)
 
 
 	/* if connected signal exists, disconnect it. */
-	if (kztab->priv->favicon_signal)
+	if (priv->favicon_signal)
 	{
 		g_signal_handler_disconnect(kztab->kz->kzfav,
-					    kztab->priv->favicon_signal);
-		kztab->priv->favicon_signal = 0;
+					    priv->favicon_signal);
+		priv->favicon_signal = 0;
 	}
 
 	if (kztab->history)
@@ -547,8 +548,11 @@ kz_tab_label_get_state (KzTabLabel *kztab)
 void
 kz_tab_label_set_width (KzTabLabel *kztab, gint width)
 {
+	KzTabLabelPrivate *priv;
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
+
 	gtk_widget_set_size_request(GTK_WIDGET(kztab), width, -1);
 	if (width < 0)
 	{
@@ -556,7 +560,7 @@ kz_tab_label_set_width (KzTabLabel *kztab, gint width)
 		gtk_widget_queue_resize(GTK_WIDGET(kztab));
 		gtk_widget_queue_resize(kztab->label);
 	}
-	kztab->priv->width = width;
+	priv->width = width;
 }
 
 void
@@ -623,10 +627,11 @@ static gboolean
 kz_tab_label_button_press (GtkWidget *widget, GdkEventButton *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (widget);
 
-	kztab->priv->start_x = event->x;
-	kztab->priv->start_y = event->y;
-	kztab->priv->moved = FALSE;
+	priv->start_x = event->x;
+	priv->start_y = event->y;
+	priv->moved = FALSE;
 
 #if 0
 	if (event->button == 3)
@@ -656,11 +661,12 @@ static gboolean
 kz_tab_label_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
-	if (abs(event->x - kztab->priv->start_x) > 2 ||
-	    abs(event->y - kztab->priv->start_y) > 2)
+	if (abs(event->x - priv->start_x) > 2 ||
+	    abs(event->y - priv->start_y) > 2)
 	{
-		kztab->priv->moved = TRUE;
+		priv->moved = TRUE;
 	}
 
 	if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event)
@@ -672,13 +678,14 @@ static gboolean
 kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
-	if (event->button == 2 && !kztab->priv->moved)
+	if (event->button == 2 && !priv->moved)
 	{
 		/* Load URL in clipboard */
 		/* return TRUE; */
 	}
-	else if (event->button == 3 && !kztab->priv->moved)
+	else if (event->button == 3 && !priv->moved)
 	{
 		kz_actions_tab_popup_menu_modal(kztab,
 						event->button,
@@ -686,9 +693,9 @@ kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
 		return TRUE;
 	}
 
-	kztab->priv->start_x = 0;
-	kztab->priv->start_y = 0;
-	kztab->priv->moved = FALSE;
+	priv->start_x = 0;
+	priv->start_y = 0;
+	priv->moved = FALSE;
 
 	if (GTK_WIDGET_CLASS(parent_class)->button_release_event)
 		return GTK_WIDGET_CLASS(parent_class)->button_release_event(widget, event);
@@ -699,22 +706,23 @@ kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
 static gboolean
 kz_tab_label_scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
-	KzTabLabel *kztab;
-	GtkNotebook *notebook;
+	KzWindow *kz;
 	gboolean retval = FALSE;
+	GtkAction *action;
 
-	kztab = KZ_TAB_LABEL(widget);
-	notebook = GTK_NOTEBOOK(kztab->kz->notebook);
+	kz = KZ_TAB_LABEL(widget)->kz;
 
 	switch (event->direction) {
 	case GDK_SCROLL_UP:
 	case GDK_SCROLL_LEFT:
-		gtk_notebook_prev_page(notebook);
+		action = gtk_action_group_get_action(kz->actions, "PrevTab");
+		gtk_action_activate(action);
 		retval = TRUE;
 		break;
 	case GDK_SCROLL_DOWN:
 	case GDK_SCROLL_RIGHT:
-		gtk_notebook_next_page(notebook);
+		action = gtk_action_group_get_action(kz->actions, "NextTab");
+		gtk_action_activate(action);
 		retval = TRUE;
 		break;
 	default:
@@ -778,7 +786,7 @@ kz_tab_label_drag_data_get (GtkWidget        *widget,
 	{
 	case TARGET_KAZEHAKASE_TAB:
 		gtk_selection_data_set(data, data->target,
-				       8, "dummy", strlen("dummy"));
+				       8, (const guchar*)"dummy", strlen("dummy"));
 		break;
 	case TARGET_NETSCAPE_URL:
 	{
@@ -815,7 +823,7 @@ kz_tab_label_drag_data_get (GtkWidget        *widget,
 			str = g_strdup (uri);
 		}
 		gtk_selection_data_set(data, data->target,
-				       8, str, strlen(str));
+				       8, (const guchar*)str, strlen(str));
 		g_free(str);
 		break;
 	}
@@ -823,7 +831,7 @@ kz_tab_label_drag_data_get (GtkWidget        *widget,
 	case TARGET_TEXT_PLAIN:
 	case TARGET_STRING:
 		gtk_selection_data_set(data, data->target,
-				       8, uri, strlen(uri));
+				       8, (const guchar*)uri, strlen(uri));
 		break;
 	default:
 		break;
@@ -867,7 +875,7 @@ kz_tab_label_drag_data_received (GtkWidget *widget,
 
 		if (data->length < 0) return;
 
-		strings = g_strsplit(data->data, "\n", 2);
+		strings = g_strsplit((const gchar*)data->data, "\n", 2);
 		kz_embed_load_url(kztab->kzembed,
 				      strings[0]);
 		g_strfreev(strings);
@@ -887,6 +895,7 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 	gboolean show_close = TRUE;
 	gboolean show_favicon = TRUE;
 	gchar *color;
+	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
 	/* width */
 	KZ_CONF_GET("Tab", "fix_width", fix, BOOL);
@@ -899,10 +908,10 @@ kz_tab_label_sync_to_profile (KzTabLabel *kztab)
 
 	/* close button */
 	KZ_CONF_GET("Tab", "show_close_button", show_close, BOOL);
-	kz_tab_label_set_show_close_button(kztab, show_close && !kztab->priv->lock);
+	kz_tab_label_set_show_close_button(kztab, show_close && !priv->lock);
 
 	/* lock button */
-	kz_tab_label_set_show_lock_button(kztab, kztab->priv->lock);
+	kz_tab_label_set_show_lock_button(kztab, priv->lock);
 
 	/* favicon */
 	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
@@ -1189,18 +1198,23 @@ cb_progress (KzEmbed *embed, KzTabLabel *kztab)
 gboolean
 kz_tab_label_get_lock (KzTabLabel *kztab)
 {
+	KzTabLabelPrivate *priv;
+
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(kztab), FALSE);
-	return kztab->priv->lock;
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
+	return priv->lock;
 }
 
 
 void
 kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 {
+	KzTabLabelPrivate *priv;
 	gboolean save = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kztab->priv->lock = lock;
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
+	priv->lock = lock;
 	kz_embed_set_lock(kztab->kzembed, lock);
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 	
@@ -1214,19 +1228,21 @@ kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 static gboolean
 cb_auto_refresh(KzTabLabel* kztab)
 {
+	KzTabLabelPrivate *priv;
 	gint auto_refresh_interval_sec = DEFAULT_AUTO_REFRESH_INTERVAL_SEC;
 
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(kztab), TRUE);
 
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 	KZ_CONF_GET("Tab", "auto_refresh_interval_sec", auto_refresh_interval_sec, INT);
 
 	kz_embed_reload(KZ_EMBED(kztab->kzembed),
 			KZ_EMBED_RELOAD_NORMAL);
 
-	if (kztab->priv->auto_refresh_id != 0)
-		g_source_remove(kztab->priv->auto_refresh_id);
+	if (priv->auto_refresh_id != 0)
+		g_source_remove(priv->auto_refresh_id);
 
-	kztab->priv->auto_refresh_id
+	priv->auto_refresh_id
 		= g_timeout_add(auto_refresh_interval_sec * 1000,
 				(GSourceFunc)cb_auto_refresh, kztab);
 	return FALSE;
@@ -1235,37 +1251,42 @@ cb_auto_refresh(KzTabLabel* kztab)
 gboolean
 kz_tab_label_get_auto_refresh (KzTabLabel *kztab)
 {
+	KzTabLabelPrivate *priv;
+	
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(kztab), FALSE);
-	return kztab->priv->auto_refresh;
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
+	return priv->auto_refresh;
 
 }
 void
 kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 {
+	KzTabLabelPrivate *priv;
 	gboolean save = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 	if(auto_refresh)
 	{
 		gint auto_refresh_interval_sec = DEFAULT_AUTO_REFRESH_INTERVAL_SEC;
 		KZ_CONF_GET("Tab", "auto_refresh_interval_sec", auto_refresh_interval_sec, INT);
 
-		if (kztab->priv->auto_refresh_id != 0)
-			g_source_remove(kztab->priv->auto_refresh_id);
+		if (priv->auto_refresh_id != 0)
+			g_source_remove(priv->auto_refresh_id);
 
-		kztab->priv->auto_refresh_id 
+		priv->auto_refresh_id 
 			= g_timeout_add(auto_refresh_interval_sec * 1000,
 					(GSourceFunc)cb_auto_refresh, kztab);
 	}
 	else
 	{
-		if (kztab->priv->auto_refresh_id != 0)
+		if (priv->auto_refresh_id != 0)
 		{
-			g_source_remove(kztab->priv->auto_refresh_id);
-			kztab->priv->auto_refresh_id = 0;
+			g_source_remove(priv->auto_refresh_id);
+			priv->auto_refresh_id = 0;
 		}
 	}
-	kztab->priv->auto_refresh = auto_refresh;
+	priv->auto_refresh = auto_refresh;
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 
 	kz_bookmark_set_auto_refresh(kztab->history, auto_refresh);
