@@ -38,7 +38,6 @@ G_BEGIN_DECLS
 
 typedef struct _KzTabLabel           KzTabLabel;
 typedef struct _KzTabLabelClass      KzTabLabelClass;
-typedef struct _KzTabLabelPriv       KzTabLabelPriv;
 
 typedef enum
 {
@@ -70,8 +69,6 @@ struct _KzTabLabel
 	GdkPixmap       *circle;
 
 	KzBookmark	*history;
-
-	KzTabLabelPriv  *priv;
 };
 
 struct _KzTabLabelClass
