@@ -45,6 +45,7 @@
 #include "utils.h"
 #include "mozilla.h"
 #include "kz-sidebar.h"
+#include "kz-feed-info.h"
 #include "kz-download-box.h"
 #include "kz-bookmark-menu.h"
 #include "kz-bookmark-tab-menu.h"
@@ -68,7 +69,8 @@ enum {
 	LAST_SIGNAL
 };
 
-struct _KzWindowPriv
+typedef struct _KzWindowPrivate	KzWindowPrivate;
+struct _KzWindowPrivate
 {
 	guint       merge_id;
 
@@ -94,6 +96,7 @@ struct _KzWindowPriv
 	GList *view_hist;
 	GNode *tab_tree;
 };
+#define KZ_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_WINDOW, KzWindowPrivate))
 
 enum {
 	STATUS_LINK_MESSAGE,
@@ -147,7 +150,6 @@ static gboolean kz_window_delete_event   (GtkWidget     *widget,
 static gboolean kz_window_key_press_event(GtkWidget     *widget,
 					  GdkEventKey   *event);
 static void     kz_window_destroy        (GtkObject     *object);
-static void     kz_window_finalize       (GObject       *object);
 
 static void     kz_window_append_tab     (KzWindow      *kz,
 					  GtkWidget     *widget,
@@ -216,6 +218,10 @@ static void     cb_notebook_switch_page       (GtkNotebook     *notebook,
 					       GtkNotebookPage *page,
 					       guint            page_num,
 					       KzWindow        *kz);
+static void     cb_notebook_switch_page_after (GtkNotebook     *notebook,
+					       GtkNotebookPage *page,
+					       guint            page_num,
+					       KzWindow        *kz);
 static void     cb_find_keyword               (GtkWidget       *widget,
 					       KzWindow        *kz);
 static gboolean cb_find_key_release           (GtkWidget       *widget,
@@ -317,8 +323,6 @@ static GList *window_list = NULL;
 KZ_OBJECT_GET_TYPE(kz_window, "KzWindow", KzWindow,
 		   kz_window_class_init, kz_window_init,
 		   GTK_TYPE_WINDOW)
-KZ_OBJECT_FINALIZE(kz_window, KzWindow)
-
 
 static void
 kz_window_class_init (KzWindowClass *klass)
@@ -363,9 +367,6 @@ kz_window_class_init (KzWindowClass *klass)
 				G_TYPE_NONE, 2,
 				GTK_TYPE_WIDGET, G_TYPE_INT);
 
-	/* GObjectClass */
-	gobject_class->finalize = kz_window_finalize;
-
 	/* GtkObject signals */
 	object_class->destroy = kz_window_destroy;
 
@@ -376,6 +377,8 @@ kz_window_class_init (KzWindowClass *klass)
 	/* KzWindow signals */
 	klass->append_tab = kz_window_append_tab;
 	klass->remove_tab = kz_window_remove_tab;
+	
+	g_type_class_add_private (gobject_class, sizeof(KzWindowPrivate));
 }
 
 
@@ -388,6 +391,7 @@ kz_window_init (KzWindow *kz)
 	GList *children, *node;
 	GtkAccelGroup *accel_group;
 	GtkSettings *settings;
+	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	g_object_set(G_OBJECT(kz), "allow-shrink", TRUE, NULL);
 
@@ -409,9 +413,10 @@ kz_window_init (KzWindow *kz)
 	kz->statusbar          = gtk_statusbar_new();
 	kz->notebook           = gtk_notebook_new();
 	kz->statusbar_area     = gtk_hbox_new(FALSE, 0);
-	kz->find_area          = kz_entry_new();
+	kz->find_area          = kz_entry_new_with_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
 	kz->find_direction     = gtk_toggle_button_new();
 	kz->dlbox              = kz_download_box_new(kz);
+	kz->feed_info          = kz_feed_info_new(kz);
 
 	kz->menu_merge         = gtk_ui_manager_new();
 	accel_group            = gtk_ui_manager_get_accel_group(kz->menu_merge);
@@ -424,25 +429,24 @@ kz_window_init (KzWindow *kz)
 	kz->popup              = kz_popup_preview_get_instance();
 	kz->popup_tablist      = NULL;
 
-	kz->priv               = g_new0(KzWindowPriv, 1);
-	kz->priv->merge_id     = 0;
-	kz->priv->status_msg   = NULL;
-	kz->priv->temp_msg     = NULL;
+	priv->merge_id     = 0;
+	priv->status_msg   = NULL;
+	priv->temp_msg     = NULL;
 
-	kz->priv->did_find     = FALSE;
+	priv->did_find     = FALSE;
 
-	kz->priv->event        = NULL;
+	priv->event        = NULL;
 
-	kz->priv->gesture      = kz_gesture_new();
-	kz->priv->start_x      = 0;
-	kz->priv->start_y      = 0;
-	kz->priv->is_gesture   = FALSE;
+	priv->gesture      = kz_gesture_new();
+	priv->start_x      = 0;
+	priv->start_y      = 0;
+	priv->is_gesture   = FALSE;
 
-	kz->priv->sidebar_was_shown = FALSE;
+	priv->sidebar_was_shown = FALSE;
 
-	kz->priv->open_hist    = NULL;
-	kz->priv->view_hist    = NULL;
-	kz->priv->tab_tree     = NULL;
+	priv->open_hist    = NULL;
+	priv->view_hist    = NULL;
+	priv->tab_tree     = NULL;
 
 	kz->tabs   = kz_bookmark_pure_folder_new();
 	kz->closed_tabs     = kz_bookmark_pure_folder_new();
@@ -453,7 +457,7 @@ kz_window_init (KzWindow *kz)
 	/* gesture */
 	{
 		kz_window_set_gesture_items(kz);
-		g_signal_connect(kz->priv->gesture,
+		g_signal_connect(priv->gesture,
 				 "stack_motion",
 				 G_CALLBACK(cb_gesture_stack_motion), kz);
 		g_signal_connect(kz_global_profile,
@@ -563,6 +567,8 @@ kz_window_init (KzWindow *kz)
 		gtk_notebook_set_scrollable(GTK_NOTEBOOK(kz->notebook), TRUE);
 		g_signal_connect(kz->notebook, "switch-page", 
 				 G_CALLBACK(cb_notebook_switch_page), kz);
+		g_signal_connect_after(kz->notebook, "switch-page",
+				 G_CALLBACK(cb_notebook_switch_page_after), kz);
 		g_signal_connect(kz->notebook, "scroll-event",
 				 G_CALLBACK(cb_notebook_scroll_event), kz);
 
@@ -617,9 +623,6 @@ kz_window_init (KzWindow *kz)
 
 		kz_entry_set_backtext(KZ_ENTRY(kz->find_area),
 				      _("Find in this page"));
-		kz_entry_set_icon_from_stock(KZ_ENTRY(kz->find_area),
-					     GTK_STOCK_FIND,
-					     GTK_ICON_SIZE_MENU);
 		g_signal_connect(kz->find_area, "activate",
 				 G_CALLBACK(cb_find_keyword), kz);
 		g_signal_connect(toggle, "toggled",
@@ -629,7 +632,7 @@ kz_window_init (KzWindow *kz)
 		gtk_object_sink(GTK_OBJECT(tips));
 		gtk_tooltips_set_tip(tips, toggle,
 				     _("Find direction"), NULL);
-		kz->priv->find_tips[0] = tips;
+		priv->find_tips[0] = tips;
 	}
 
 	/* find entry */
@@ -652,17 +655,35 @@ kz_window_init (KzWindow *kz)
 		gtk_object_sink(GTK_OBJECT(tips));
 		gtk_tooltips_set_tip(tips, kz->find_area,
 				     _("Find a word or phrase in this page"), NULL);
-		kz->priv->find_tips[1] = tips;
+		priv->find_tips[1] = tips;
 	}
 
+	/* feed info */
+	{
+		gtk_widget_set_size_request(kz->feed_info, 64, -1);
+		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(kz->feed_info),
+						  TRUE);
+		gtk_box_pack_end(GTK_BOX(kz->statusbar_area), kz->feed_info,
+				 FALSE, FALSE, 0);
+		gtk_widget_show(kz->feed_info);
+	}
+
+	/* separator */
+	{
+		vseparator = gtk_vseparator_new();
+		gtk_box_pack_end(GTK_BOX(kz->statusbar_area), vseparator,
+				 FALSE, FALSE, 0);
+		gtk_widget_show(vseparator);
+	}
+	
 	/* download box */
 	{
 		gtk_widget_set_size_request(kz->dlbox, 200, -1);
 		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(kz->dlbox),
-						  TRUE);
+						  FALSE);
 		gtk_statusbar_push(GTK_STATUSBAR(kz->dlbox), 0,
 				   _("Drop link to download"));
-		gtk_box_pack_end(GTK_BOX(kz->statusbar_area), kz->dlbox,  
+		gtk_box_pack_end(GTK_BOX(kz->statusbar_area), kz->dlbox,
 				 FALSE, FALSE, 0);
 		gtk_widget_show(kz->dlbox);
 	}
@@ -680,7 +701,7 @@ kz_window_init (KzWindow *kz)
 				   FALSE, FALSE, 0);
 		gtk_widget_show(vseparator);
 	}
-
+	
 	/* from Galeon-1.3.18 */
 	/* initialize the listener for the key theme */
 	settings = gtk_settings_get_default();
@@ -749,9 +770,12 @@ kz_window_sync_ui_level (KzWindow *kz)
 	gchar *ui_file;
 	GtkWidget *widget;
 	KzEmbed *kzembed;
+	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
+
 	switch (kz_ui_level()) 
 	{
 	case KZ_UI_LEVEL_CUSTOM:
@@ -780,17 +804,16 @@ kz_window_sync_ui_level (KzWindow *kz)
 	if (!gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)))
 		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 
-	if (kz->priv->merge_id != 0)
+	if (priv->merge_id != 0)
 	{
-		gtk_ui_manager_remove_ui(kz->menu_merge, kz->priv->merge_id);
-		kz->priv->merge_id = 0;
+		gtk_ui_manager_remove_ui(kz->menu_merge, priv->merge_id);
+		priv->merge_id = 0;
 		kz_actions_remove_smartbookmarks(kz, kz_bookmarks->smarts);
         	gtk_ui_manager_ensure_update(kz->menu_merge);
 	}
 
-	kz->priv->merge_id
-		= gtk_ui_manager_add_ui_from_file(kz->menu_merge,
-						  ui_file, &err);
+	priv->merge_id = gtk_ui_manager_add_ui_from_file(kz->menu_merge,
+						         ui_file, &err);
 	if (err)
 	{
 		g_warning("%s", err->message);
@@ -951,9 +974,10 @@ kz_window_open_new_tab_at_tail (KzWindow *kz, const gchar *url)
 	KzEmbed *kzembed;
 	KzTabLabel *kztab;
 	GNode *node;
-
+	KzWindowPrivate *priv;
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	kzembed = kz_window_create_embed(kz, url);
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
@@ -965,15 +989,15 @@ kz_window_open_new_tab_at_tail (KzWindow *kz, const gchar *url)
 	kz_window_set_embed_callbacks(kz, kzembed);
 
 	/* add to this window's history */
-	kz->priv->open_hist = g_list_prepend(kz->priv->open_hist, kzembed);
+	priv->open_hist = g_list_prepend(priv->open_hist, kzembed);
 
 	/* root node */
-	if (!kz->priv->tab_tree)
-		kz->priv->tab_tree = g_node_new(NULL);
+	if (!priv->tab_tree)
+		priv->tab_tree = g_node_new(NULL);
 
 	/* insret node */
 	node = g_node_new(kzembed);
-	g_node_append(kz->priv->tab_tree, node);
+	g_node_append(priv->tab_tree, node);
 
 	kz_bookmark_append(kz->tabs, kztab->history);
 
@@ -1049,10 +1073,12 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 	KzTabLabel *kztab, *sibtab;
 	GNode *node, *parent_node;
 	gint pos;
-
+	KzWindowPrivate *priv;
+	
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 	if (url) g_return_val_if_fail(g_utf8_validate(url, strlen(url), NULL), NULL);
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	kzembed = kz_window_create_embed(kz, url);
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
@@ -1078,20 +1104,20 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 	kz_window_set_embed_callbacks(kz, kzembed);
 
 	/* add to this window's history */
-	kz->priv->open_hist = g_list_prepend(kz->priv->open_hist, kzembed);
+	priv->open_hist = g_list_prepend(priv->open_hist, kzembed);
 
 	/* root node */
-	if (!kz->priv->tab_tree)
-		kz->priv->tab_tree = g_node_new(NULL);
+	if (!priv->tab_tree)
+		priv->tab_tree = g_node_new(NULL);
 
 	/* insret node */
 	node = g_node_new(kzembed);
-	parent_node = g_node_find(kz->priv->tab_tree,
+	parent_node = g_node_find(priv->tab_tree,
 				  G_IN_ORDER, G_TRAVERSE_ALL, parent);
 	if (parent_node)
 		g_node_append(parent_node, node);
 	else
-		g_node_append(kz->priv->tab_tree, node);
+		g_node_append(priv->tab_tree, node);
 
 	g_signal_emit(kz, kz_window_signals[APPEND_TAB_SIGNAL],
 		      0, kzembed, parent);
@@ -1106,10 +1132,12 @@ kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 	gchar *ret_page = NULL;
 	KzEmbed *next = NULL;
 	KzTabLabel *kztab = NULL;
-
+	KzWindowPrivate *priv;
+	
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
 							widget));
 	if (kztab && kz_tab_label_get_lock(kztab)) {
@@ -1127,8 +1155,8 @@ kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 	{
 		GList *node;
 
-		if (kz->priv->view_hist &&
-		    (node = g_list_next(kz->priv->view_hist)))
+		if (priv->view_hist &&
+		    (node = g_list_next(priv->view_hist)))
 		{
 			next = node->data;
 		}
@@ -1137,7 +1165,7 @@ kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 	{
 		GList *node;
 
-		node = kz->priv->open_hist;
+		node = priv->open_hist;
 		for (; node && !next; node = g_list_next(node))
 		{
 			if (KZ_WINDOW_CURRENT_PAGE(kz) != node->data)
@@ -1251,15 +1279,18 @@ kz_window_move_tab (KzWindow *kz, GtkWidget *widget)
 	KzEmbed *kzembed;
 	KzTabLabel *new_kztab;
 	GNode *node, *child;
+	KzWindowPrivate *priv, *src_priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	g_return_if_fail(KZ_IS_EMBED(widget));
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	kzembed = KZ_EMBED(widget);
 
 	src_kz = KZ_WINDOW(kz_window_get_from_tab(widget));
 	if (!src_kz) return;
 
+	src_priv = KZ_WINDOW_GET_PRIVATE (src_kz);
 	label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
 					   widget);
 
@@ -1286,16 +1317,16 @@ kz_window_move_tab (KzWindow *kz, GtkWidget *widget)
 	kz_window_set_embed_callbacks(kz, kzembed);
 
 	/* remove view_hist */
-	src_kz->priv->view_hist = g_list_remove(src_kz->priv->view_hist,
+	src_priv->view_hist = g_list_remove(src_priv->view_hist,
 						kzembed);
 	/* move open_hist */
-	src_kz->priv->open_hist = g_list_remove(src_kz->priv->open_hist,
+	src_priv->open_hist = g_list_remove(src_priv->open_hist,
 						kzembed);
-	kz->priv->open_hist = g_list_prepend(kz->priv->open_hist,
+	priv->open_hist = g_list_prepend(priv->open_hist,
 					     kzembed);
 
 	/* move tab tree */
-	node = g_node_find(src_kz->priv->tab_tree,
+	node = g_node_find(src_priv->tab_tree,
 			   G_IN_ORDER, G_TRAVERSE_ALL, widget);
 	if (node)
 	{
@@ -1305,16 +1336,16 @@ kz_window_move_tab (KzWindow *kz, GtkWidget *widget)
 		{
 			GNode *next = g_node_next_sibling(child);
 			g_node_unlink(child);
-			g_node_append(src_kz->priv->tab_tree,
+			g_node_append(src_priv->tab_tree,
 				      child);
 			child = next;
 		}
 
 		/* move node */
 		g_node_unlink(node);
-		if (!kz->priv->tab_tree)
-			kz->priv->tab_tree = g_node_new(NULL);
-		g_node_append(kz->priv->tab_tree, node);
+		if (!priv->tab_tree)
+			priv->tab_tree = g_node_new(NULL);
+		g_node_append(priv->tab_tree, node);
 		
 		g_signal_emit(src_kz,
 			      kz_window_signals[REMOVE_TAB_SIGNAL],
@@ -1470,29 +1501,30 @@ static void
 kz_window_destroy (GtkObject *object)
 {
 	KzWindow *kz = KZ_WINDOW(object);
+	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		GTK_OBJECT_CLASS (parent_class)->destroy(object);
 
 	window_list = g_list_remove(window_list, kz);
 
-	if (kz->priv->gesture)
-		g_object_unref(kz->priv->gesture);
-	kz->priv->gesture = NULL;
+	if (priv->gesture)
+		g_object_unref(priv->gesture);
+	priv->gesture = NULL;
 
-	g_free(kz->priv->temp_msg);
-        kz->priv->temp_msg = NULL;
+	g_free(priv->temp_msg);
+        priv->temp_msg = NULL;
 
-	if (kz->priv->event)
-		kz_embed_event_free((KzEmbedEvent *) kz->priv->event);
-	kz->priv->event = NULL;
+	if (priv->event)
+		kz_embed_event_free((KzEmbedEvent *) priv->event);
+	priv->event = NULL;
 
-	if (kz->priv->find_tips[0])
-		g_object_unref(kz->priv->find_tips[0]);
-	kz->priv->find_tips[0] = NULL;
-	if (kz->priv->find_tips[1])
-		g_object_unref(kz->priv->find_tips[1]);
-	kz->priv->find_tips[1] = NULL;
+	if (priv->find_tips[0])
+		g_object_unref(priv->find_tips[0]);
+	priv->find_tips[0] = NULL;
+	if (priv->find_tips[1])
+		g_object_unref(priv->find_tips[1]);
+	priv->find_tips[1] = NULL;
 
 	if (kz->menu_merge)
 		g_object_unref(kz->menu_merge);
@@ -1586,8 +1618,10 @@ kz_window_set_gesture_items (KzWindow *kz)
 	GtkAction *action;
 	KzGestureItems *items;
 	GList *list, *node;
+	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	items = kz_gesture_items_new();
 
@@ -1616,7 +1650,7 @@ kz_window_set_gesture_items (KzWindow *kz)
 		g_free(gesture);
 	}
 
-	kz_gesture_set_items(kz->priv->gesture, items);
+	kz_gesture_set_items(priv->gesture, items);
 	kz_gesture_items_unref(items);
 
 	g_list_free(list);
@@ -1675,8 +1709,10 @@ kz_window_store_state (KzWindow *kz)
 	gint i, client_x, client_y, width, height;
 	const gchar *label;
 	gboolean active;
+	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	kz_profile_set_save_each_time(kz_global_profile, FALSE);
 	/*
@@ -1707,7 +1743,7 @@ kz_window_store_state (KzWindow *kz)
 			 * not set correctly(maybe both 1), so we should use 
 			 * sidebar or notebook width according to sidebar position.
 			 */
-			if (kz->priv->sidebar_was_shown)
+			if (priv->sidebar_was_shown)
 			{
 				if (!strcmp(positems[i].pos, "top"))
 					width = kz->sidebar->allocation.height;
@@ -2068,16 +2104,20 @@ kz_window_get_tab_label (KzWindow *kz, GtkWidget *widget)
 GNode *
 kz_window_get_tree (KzWindow *kz)
 {
+	KzWindowPrivate *priv;
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
-	return kz->priv->tab_tree;
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
+	return priv->tab_tree;
 }
 
 
 const KzEmbedEventMouse *
 kz_window_get_mouse_event_info (KzWindow *kz)
 {
+	KzWindowPrivate *priv;
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
-	return kz->priv->event;
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
+	return priv->event;
 }
 
 
@@ -2323,6 +2363,7 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	GtkAction *action;
 	KzTabLabel *tab;
 	gboolean focus;
+	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	g_return_if_fail(KZ_IS_EMBED(cur));
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
@@ -2392,8 +2433,8 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	kz_actions_set_sensitive(kz, kzembed);
 	kz_actions_set_tab_sensitive(kz, kzembed);
 
-	kz->priv->view_hist = g_list_remove(kz->priv->view_hist, kzembed);
-	kz->priv->view_hist = g_list_prepend(kz->priv->view_hist, kzembed);
+	priv->view_hist = g_list_remove(priv->view_hist, kzembed);
+	priv->view_hist = g_list_prepend(priv->view_hist, kzembed);
 
 	/* set current_position in history */
 	if (!kz->is_closing_all && !is_restoring_session)
@@ -2403,6 +2444,14 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	}
 }
 
+static void
+cb_notebook_switch_page_after (GtkNotebook *notebook, GtkNotebookPage *page,
+			       guint page_num, KzWindow *kz)
+{
+	kz_feed_info_change_state(KZ_FEED_INFO(kz->feed_info));
+}
+
+
 static void
 search_found(GtkWidget* widget, KzWindow *kz)
 {
@@ -2451,6 +2500,7 @@ cb_find_keyword (GtkWidget *widget, KzWindow *kz)
 	GtkToggleButton *toggle = GTK_TOGGLE_BUTTON(kz->find_direction);
 	const gchar *text = gtk_entry_get_text(GTK_ENTRY(widget));
 	gboolean back = gtk_toggle_button_get_active(toggle);
+	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	if (!embed) return;
 
@@ -2460,8 +2510,8 @@ cb_find_keyword (GtkWidget *widget, KzWindow *kz)
 		return;
 	}
 
-	kz->priv->did_find = kz_embed_find(embed, text, back);
-	if(kz->priv->did_find)
+	priv->did_find = kz_embed_find(embed, text, back);
+	if(priv->did_find)
 		search_found(widget, kz);
 	else
 		search_not_found(widget, kz);
@@ -2473,12 +2523,14 @@ cb_find_key_release (GtkWidget *widget, GdkEventKey *event, KzWindow *kz)
 	KzEmbed *embed ;
 	GtkToggleButton *toggle;
 	const gchar *text;
+	KzWindowPrivate *priv;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
 	embed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
 	if (!embed) return FALSE;
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	toggle = GTK_TOGGLE_BUTTON(kz->find_direction);
 	text = gtk_entry_get_text(GTK_ENTRY(widget));
 	if (!text || !(*text))
@@ -2493,7 +2545,7 @@ cb_find_key_release (GtkWidget *widget, GdkEventKey *event, KzWindow *kz)
 		{
 			gboolean back;
 			back = !gtk_toggle_button_get_active(toggle);
-			kz->priv->did_find
+			priv->did_find
 				= kz_embed_find(embed, text, back);
 		}
 	}
@@ -2503,12 +2555,12 @@ cb_find_key_release (GtkWidget *widget, GdkEventKey *event, KzWindow *kz)
 		{
 			gboolean back;
 			back = gtk_toggle_button_get_active(toggle);
-			kz->priv->did_find
+			priv->did_find
 				= kz_embed_incremental_search(embed, text, back);
 		}
 	}
 
-	if(kz->priv->did_find)
+	if(priv->did_find)
 		search_found(widget, kz);
 	else
 		search_not_found(widget, kz);
@@ -2573,41 +2625,21 @@ static gboolean
 cb_notebook_scroll_event (GtkWidget *widget, GdkEventScroll *event,
 			  KzWindow *kz)
 {
-	GtkNotebook *notebook;
-	gboolean circulation = FALSE;
+	GtkAction *action;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
-	notebook = GTK_NOTEBOOK(kz->notebook);
-	KZ_CONF_GET("Tab", "wheel_circulation", circulation, BOOL);
-
 	switch (event->direction) {
 	case GDK_SCROLL_UP:
 	case GDK_SCROLL_LEFT:
-		if (circulation &&
-		    gtk_notebook_get_current_page(notebook) == 0) 
-		{
-			gtk_notebook_set_current_page
-				(notebook, gtk_notebook_get_n_pages(notebook) - 1);
-		}
-		else
-		{
-			gtk_notebook_prev_page(notebook);
-		}
+		action = gtk_action_group_get_action(kz->actions, "PrevTab");
+		gtk_action_activate(action);
 		return TRUE;
 		break;
 	case GDK_SCROLL_DOWN:
 	case GDK_SCROLL_RIGHT:
-		if (circulation && 
-		    gtk_notebook_get_current_page(notebook) == 
-		    gtk_notebook_get_n_pages(notebook) - 1) 
-		{
-			gtk_notebook_set_current_page(notebook, 0);
-		}
-		else
-		{
-			gtk_notebook_next_page(notebook);
-		}
+		action = gtk_action_group_get_action(kz->actions, "NextTab");
+		gtk_action_activate(action);
 		return TRUE;
 		break;
 	default:
@@ -2622,9 +2654,11 @@ cb_notebook_scroll_event (GtkWidget *widget, GdkEventScroll *event,
 static void
 cb_sidebar_map (GtkWidget *widget, GtkToggleAction *action)
 {
+	KzWindowPrivate *priv;
 	g_return_if_fail(GTK_IS_TOGGLE_ACTION(action));
 
-	KZ_SIDEBAR(widget)->kz->priv->sidebar_was_shown = TRUE;
+	priv = KZ_WINDOW_GET_PRIVATE (KZ_SIDEBAR(widget)->kz);
+	priv->sidebar_was_shown = TRUE;
 
 	gtk_toggle_action_set_active(action, TRUE);
 }
@@ -2713,6 +2747,8 @@ cb_embed_load_started (KzEmbed *embed, KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	kz_actions_set_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+
+	kz_feed_info_change_state(KZ_FEED_INFO(kz->feed_info));
 }
 
 
@@ -2722,6 +2758,8 @@ cb_embed_load_finished (KzEmbed *embed, KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	kz_actions_set_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+
+	kz_feed_info_change_state(KZ_FEED_INFO(kz->feed_info));
 }
 
 
@@ -2741,21 +2779,23 @@ cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
 {
 	KzEmbed *kzembed;
 	GNode *node, *child;
+	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(kz->priv->tab_tree);
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
+	g_return_if_fail(priv->tab_tree);
 
 	kzembed = KZ_EMBED(obj);
 
 	kz_window_unset_cur_embed_callbacks(kz, kzembed);
 	kz_window_unset_embed_callbacks(kz, kzembed);
 
-	kz->priv->view_hist = g_list_remove(kz->priv->view_hist, kzembed);
-	kz->priv->open_hist = g_list_remove(kz->priv->open_hist, kzembed);
+	priv->view_hist = g_list_remove(priv->view_hist, kzembed);
+	priv->open_hist = g_list_remove(priv->open_hist, kzembed);
 
 	gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar), STATUS_LINK_MESSAGE);
 
-	node = g_node_find(kz->priv->tab_tree,
+	node = g_node_find(priv->tab_tree,
 			   G_IN_ORDER, G_TRAVERSE_ALL, kzembed);
 	/* probably destroy handler is already called */
 	if (!node) return;
@@ -2766,7 +2806,7 @@ cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
 	{
 		GNode *next = g_node_next_sibling(child);
 		g_node_unlink(child);
-		g_node_append(kz->priv->tab_tree, child);
+		g_node_append(priv->tab_tree, child);
 		child = next;
 	}
 	
@@ -2870,8 +2910,6 @@ cb_embed_dom_mouse_click (KzEmbed *embed, KzEmbedEventMouse *event,
 			/* what about uri list? */
 			if (action)
 				gtk_action_activate(action);
-
-			break;
 		}
 		break;
 	default:
@@ -2915,14 +2953,16 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 {
 	gint button;
      	glong type;
+	KzWindowPrivate *priv;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 	type = event->cinfo.context;
 	button = event->button;
-	if (kz->priv->event)
-		kz_embed_event_free((KzEmbedEvent *) kz->priv->event);
-	kz->priv->event = (KzEmbedEventMouse *) kz_embed_event_copy((KzEmbedEvent *) event);
+	if (priv->event)
+		kz_embed_event_free((KzEmbedEvent *) priv->event);
+	priv->event = (KzEmbedEventMouse *) kz_embed_event_copy((KzEmbedEvent *) event);
 
 	switch (button)
 	{
@@ -2984,7 +3024,7 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 		gboolean use_gesture = TRUE;
 		gint x, y, win_x, win_y, win_x_pos, win_y_pos;
 
-		kz->priv->is_gesture = FALSE;
+		priv->is_gesture = FALSE;
 		KZ_CONF_GET("Gesture", "use_gesture", use_gesture, BOOL);
 		if (!use_gesture)
 		{
@@ -2999,8 +3039,8 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 
 		gtk_widget_get_pointer (GTK_WIDGET(kzembed), &x, &y);
 
-		kz->priv->start_x = x;
-		kz->priv->start_y = y;
+		priv->start_x = x;
+		priv->start_y = y;
 
 		gdk_window_get_root_origin (GTK_WIDGET(kzembed)->window,
 					    &win_x, &win_y);
@@ -3009,7 +3049,7 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 
 		gtk_widget_get_pointer (GTK_WIDGET(kzembed),
 					&x, &y);
-		kz_gesture_start(kz->priv->gesture, 0, x, y);
+		kz_gesture_start(priv->gesture, 0, x, y);
 
 		if (!cursor) cursor = gdk_cursor_new (GDK_HAND1);
 		gdk_pointer_grab (GTK_WIDGET(kz)->window,
@@ -3087,24 +3127,26 @@ cb_window_motion_notify_event (GtkWidget *widget,
 	KzWindow *kz;
 	gint x, y;
 	/* GdkModifierType state; */
+	KzWindowPrivate *priv;
 
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
 
 	kz = KZ_WINDOW(widget);
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 
 	/*gdk_window_get_pointer (GTK_WIDGET(kzembed)->window, &x, &y, &state);*/
 	gtk_widget_get_pointer (GTK_WIDGET(kzembed), &x, &y);
 
-	if (abs(x - kz->priv->start_x) > 2 ||
-	    abs(y - kz->priv->start_y) > 2)
+	if (abs(x - priv->start_x) > 2 ||
+	    abs(y - priv->start_y) > 2)
 	{
-		kz->priv->is_gesture = TRUE;
+		priv->is_gesture = TRUE;
 	}
 
-	if (kz_gesture_is_started(kz->priv->gesture))
+	if (kz_gesture_is_started(priv->gesture))
 	{
 		/* if(state == GDK_BUTTON3_MASK) */
-		kz_gesture_update_position(kz->priv->gesture, x, y);
+		kz_gesture_update_position(priv->gesture, x, y);
 		/*
 		else
 		{
@@ -3112,16 +3154,16 @@ cb_window_motion_notify_event (GtkWidget *widget,
 			{
 				gdk_pointer_ungrab (gtk_get_current_event_time ());
 			}
-			kz_gesture_perform(kz->priv->gesture);
+			kz_gesture_perform(priv->gesture);
 			gtk_statusbar_push(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
 				   STATUS_GESTURE, "");
-			kz->priv->start_x = 0;
-			kz->priv->start_y = 0;
-			kz->priv->is_gesture = FALSE;
+			priv->start_x = 0;
+			priv->start_y = 0;
+			priv->is_gesture = FALSE;
 
-			if (kz->priv->event)
-				kz_embed_event_free((KzEmbedEvent *) kz->priv->event);
-			kz->priv->event = NULL;
+			if (priv->event)
+				kz_embed_event_free((KzEmbedEvent *) priv->event);
+			priv->event = NULL;
 		}
 		*/
 	}
@@ -3139,39 +3181,41 @@ cb_window_button_release_event (GtkWidget *widget,
 {
 	KzWindow *kz;
 	gboolean retval = FALSE;
+	KzWindowPrivate *priv;
 
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
 
 	kz = KZ_WINDOW(widget);
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
 
         if (gdk_pointer_is_grabbed ())
         {
                 gdk_pointer_ungrab (gtk_get_current_event_time ());
         }
 
-	if (kz_gesture_is_started(kz->priv->gesture))
+	if (kz_gesture_is_started(priv->gesture))
 	{
 		if (event->button == 3)
-			kz_gesture_perform(kz->priv->gesture);
+			kz_gesture_perform(priv->gesture);
 		else
-			kz_gesture_cancel(kz->priv->gesture);
+			kz_gesture_cancel(priv->gesture);
 		gtk_statusbar_pop(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
 				  STATUS_GESTURE);
 	}
 
-	if (!kz->priv->is_gesture && event->button == 3)
+	if (!priv->is_gesture && event->button == 3)
 	{
 		kz_actions_popup_menu_modal (kz, event->button, event->time);
 		retval = TRUE;
 	}
 
-	kz->priv->start_x = 0;
-	kz->priv->start_y = 0;
-	kz->priv->is_gesture = FALSE;
+	priv->start_x = 0;
+	priv->start_y = 0;
+	priv->is_gesture = FALSE;
 
-	if (kz->priv->event)
-		kz_embed_event_free((KzEmbedEvent *) kz->priv->event);
-	kz->priv->event = NULL;
+	if (priv->event)
+		kz_embed_event_free((KzEmbedEvent *) priv->event);
+	priv->event = NULL;
 
 	return retval;
 }
@@ -3251,12 +3295,13 @@ cb_focus_out_event            (GtkWidget       *widget,
 			       GdkEventFocus   *event,
 			       KzWindow        *kz)
 {
+	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 	if(!event->in) 
 	{
 		/* focus is out */
-		if (kz_gesture_is_started(kz->priv->gesture))
+		if (kz_gesture_is_started(priv->gesture))
 		{
-			kz_gesture_cancel(kz->priv->gesture);
+			kz_gesture_cancel(priv->gesture);
 			gtk_statusbar_pop(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
 					  STATUS_GESTURE);
 		}
