@@ -42,7 +42,6 @@ G_BEGIN_DECLS
 
 typedef struct _KzWindow	KzWindow;
 typedef struct _KzWindowClass	KzWindowClass;
-typedef struct _KzWindowPriv	KzWindowPriv;
 
 struct _KzWindow
 {
@@ -69,6 +68,7 @@ struct _KzWindow
 	GtkWidget  *find_direction;
 	GtkWidget  *statusbar;
 	GtkWidget  *dlbox;
+	GtkWidget  *feed_info;
 
 	/* other objects */
 	GtkActionGroup *actions;
@@ -87,8 +87,6 @@ struct _KzWindow
 	GtkWidget *popup_tablist;
 
 	gboolean   is_closing_all;
-
-	KzWindowPriv *priv;
 };
 
 struct _KzWindowClass
