@@ -423,7 +423,7 @@ cb_selection_received(GtkWidget *widget,
 
 
 		/* open URL */
-		argv = split_string(data->data, &argc);
+		argv = split_string((const char*)data->data, &argc);
 
 		retval = g_option_context_parse (context, &argc, &argv, &error);
 		if(retval)
