@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozillaPrivate.cpp,v 1.7 2005/06/27 23:34:14 ikezoe Exp $
+ *  $Id: MozillaPrivate.cpp,v 1.9 2005/07/22 08:55:45 ikezoe Exp $
  */
 
 /*
@@ -29,26 +29,31 @@
 
 #include "MozillaPrivate.h"
 
-#include <nsIPrintSettingsService.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#include <nsIComponentManager.h>
+#include <nsIPrintSettingsService.h>
+#include <nsIPrintOptions.h>
+#include <nsWeakPtr.h>
+#include <nsPromiseFlatString.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsISimpleEnumerator.h>
 #include <nsISupportsPrimitives.h>
-#include <nsPromiseFlatString.h>
-#include <nsIPrintOptions.h>
 #include <nsIContent.h>
 #include <nsIDOMHTMLDocument.h>
 #include <nsIDOMHTMLElement.h>
+#include <nsIWidget.h>
+#include <nsPIDOMWindow.h>
+#include <nsIChromeEventHandler.h>
+#include <nsIDOMWindowInternal.h>
+#include <nsIDOMDocument.h>
+#include <nsIDocument.h>
 #ifdef HAVE_NSIPRESCONTEXT_H
 #include <nsIPresContext.h>
 #else
 #include <nsPresContext.h>
 #endif
-#include <nsIWidget.h>
-#include <nsProfileDirServiceProvider.h>
-#include <nsPIDOMWindow.h>
-#include <nsIChromeEventHandler.h>
-#include <nsIDOMWindowInternal.h>
-#include <nsIDOMDocument.h>
+
 
 /* IMPORTANT. Put only code that use internal mozilla strings (nsAutoString for
  * example) in this file. Note that you cannot use embed strings here,
@@ -243,14 +248,21 @@ MozillaPrivate::GetEventReceiver (nsIDOMWindow *domWindow, nsIDOMEventReceiver *
 	nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
   	// and the root window for that DOM window
   	nsCOMPtr<nsIDOMWindowInternal> rootWindow;
-  	domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
+#ifndef MOZ_NSPIDOMWINDOW_RETURNNSPIDOMWINDOW_ 
+	domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
+#else
+  	rootWindow = domWindowPrivate->GetPrivateRoot();
+#endif
   
   	nsCOMPtr<nsPIDOMWindow> piWin;
 	piWin = do_QueryInterface(rootWindow);
 
 	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
+#ifndef MOZ_NSPIDOMWINDOW_RETURNNSPIDOMWINDOW_ 
 	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-
+#else
+	chromeHandler = piWin->GetChromeEventHandler();
+#endif
 	nsCOMPtr<nsIDOMEventReceiver> er;
 	er = do_QueryInterface(chromeHandler);
 
@@ -282,72 +294,3 @@ MozillaPrivate::GetURIForDOMWindow(nsIDOMWindow* aDOMWindow)
 	return g_strdup(str.get());
 }
 
-
-void    *MozillaEmbedPrivate::sProfileDirServiceProvider = nsnull;
-nsIPref *MozillaEmbedPrivate::sPrefs       = nsnull;
-
-MozillaEmbedPrivate::MozillaEmbedPrivate(void)
-{
-}
-MozillaEmbedPrivate::~MozillaEmbedPrivate()
-{
-}
-
-nsresult
-MozillaEmbedPrivate::StartupProfile(const char *sProfileDir, const char *sProfileName)
-{
-  // initialize profiles
-  if (sProfileDir && sProfileName) {
-    nsresult rv;
-    nsCOMPtr<nsILocalFile> profileDir;
-    NS_NewNativeLocalFile(nsDependentCString(sProfileDir), PR_TRUE,
-                          getter_AddRefs(profileDir));
-    if (!profileDir)
-      return NS_ERROR_FAILURE;
-    rv = profileDir->AppendNative(nsDependentCString(sProfileName));
-    if (NS_FAILED(rv))
-      return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsProfileDirServiceProvider> locProvider;
-    NS_NewProfileDirServiceProvider(PR_TRUE, getter_AddRefs(locProvider));
-    if (!locProvider)
-      return NS_ERROR_FAILURE;
-    rv = locProvider->Register();
-    if (NS_FAILED(rv))
-      return rv;
-    rv = locProvider->SetProfileDir(profileDir);
-    if (NS_FAILED(rv))
-      return rv;
-    
-    nsProfileDirServiceProvider *provider;
-    // Keep a ref so we can shut it down.
-    NS_ADDREF(provider = locProvider);
-    sProfileDirServiceProvider = (void*)provider;
-
-    // get prefs
-    nsCOMPtr<nsIPref> pref;
-    pref = do_GetService(NS_PREF_CONTRACTID);
-    if (!pref)
-      return NS_ERROR_FAILURE;
-    sPrefs = pref.get();
-    NS_ADDREF(sPrefs);
-  }
-  return NS_OK;
-}
-
-/* static */
-void
-MozillaEmbedPrivate::ShutdownProfile(void)
-{
-  if (sProfileDirServiceProvider) {
-    nsProfileDirServiceProvider *provider = (nsProfileDirServiceProvider *)sProfileDirServiceProvider;
-    provider->Shutdown();
-    NS_RELEASE(provider);
-    sProfileDirServiceProvider = 0;
-  }
-  if (sPrefs) {
-    NS_RELEASE(sPrefs);
-    sPrefs = 0;
-  }
-}
-
