@@ -89,13 +89,6 @@
 #include <nsILocalFile.h>
 #include <nsIDOM3Node.h>
 
-#define N_LINKS 7
-
-static gchar *nav_names[N_LINKS] =
-{
- "NEXT", "PREV", "INDEX", "CONTENTS", "START", "RSS", "ALTERNATE"
-};
-
 typedef struct _KzMozEmbedPrivate	KzMozEmbedPrivate;
 struct _KzMozEmbedPrivate
 {
@@ -107,7 +100,7 @@ struct _KzMozEmbedPrivate
 	gboolean lock;
 
 	/* for navigation link */
-	gchar *nav_links[N_LINKS];
+	GList *nav_links[KZ_EMBED_LINK_GUARD];
 
 #ifdef USE_MIGEMO
 	gchar *migemo_keyword;
@@ -232,10 +225,17 @@ static gboolean     kz_moz_embed_can_go_nav_link       (KzEmbed      *kzembed,
 							KzEmbedNavLink link);
 static void         kz_moz_embed_go_nav_link           (KzEmbed      *kzembed,
 							KzEmbedNavLink link);
-static void         kz_moz_embed_set_nav_link          (KzEmbed      *kzembed,
-							const gchar  *name,
-							const gchar  *link);
-static const gchar *kz_moz_embed_get_nav_link          (KzEmbed      *kzembed,
+static void         kz_moz_embed_append_nav_link       (KzEmbed      *kzembed,
+							KzEmbedNavLink link,
+							KzNavi       *navi);
+static void         kz_moz_embed_set_nth_nav_link      (KzEmbed      *kzembed,
+							KzEmbedNavLink link,
+							KzNavi       *navi,
+							guint         n);
+static KzNavi      *kz_moz_embed_get_nth_nav_link      (KzEmbed      *kzembed,
+							KzEmbedNavLink link,
+							guint         n);
+static GList       *kz_moz_embed_get_nav_links         (KzEmbed      *kzembed,
 							KzEmbedNavLink link);
 static void         kz_moz_embed_go_history_index      (KzEmbed      *kzembed,
 							gint          index);
@@ -348,8 +348,12 @@ kz_moz_embed_iface_init (KzEmbedIFace *iface)
 	iface->can_go_forward         = kz_moz_embed_can_go_forward;
 	iface->can_go_nav_link        = kz_moz_embed_can_go_nav_link;
 	iface->go_nav_link            = kz_moz_embed_go_nav_link;
-	iface->set_nav_link           = kz_moz_embed_set_nav_link;
-	iface->get_nav_link           = kz_moz_embed_get_nav_link;
+	iface->append_nav_link        = kz_moz_embed_append_nav_link;
+	iface->set_nav_link           = NULL;
+	iface->set_nth_nav_link       = kz_moz_embed_set_nth_nav_link;
+	iface->get_nav_link           = NULL;
+	iface->get_nth_nav_link       = kz_moz_embed_get_nth_nav_link;
+	iface->get_nav_links          = kz_moz_embed_get_nav_links;
 	iface->go_history_index       = kz_moz_embed_go_history_index;
 	iface->do_command             = kz_moz_embed_do_command;
 	iface->can_do_command         = kz_moz_embed_can_do_command;
@@ -497,7 +501,7 @@ kz_moz_embed_init (KzMozEmbed *kzembed)
 	priv->total_requests = 0;
 	priv->cur_requests   = 0;
 	
-	for (i =0; i < N_LINKS; i++)
+	for (i = 0; i < KZ_EMBED_LINK_GUARD; i++)
 	{
 		priv->nav_links[i] = NULL;
 	}
@@ -603,6 +607,12 @@ kz_moz_embed_unrealize (GtkWidget *widget)
 		GTK_WIDGET_CLASS(parent_class)->unrealize(widget);
 }
 
+static void
+nav_link_elem_free(gpointer object, gpointer unused)
+{
+	if (object)
+		g_object_unref(G_OBJECT(object));
+}
 
 static void
 kz_moz_embed_navigation_link_free(KzMozEmbed *kzembed)
@@ -610,11 +620,12 @@ kz_moz_embed_navigation_link_free(KzMozEmbed *kzembed)
 	gint i;
 	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
 	
-	for (i =0; i < N_LINKS; i++)
+	for (i = 0; i < KZ_EMBED_LINK_GUARD; i++)
 	{
 		if (priv->nav_links[i])
 		{
-			g_free(priv->nav_links[i]);
+			g_list_foreach(priv->nav_links[i], nav_link_elem_free, NULL);
+			g_list_free(priv->nav_links[i]);
 			priv->nav_links[i] = NULL;
 		}
 	}
@@ -926,6 +937,10 @@ kz_moz_embed_net_start (GtkMozEmbed *embed)
 		return;
 
 	kzembed->is_loading = TRUE;
+
+	/* First free previous link */
+	kz_moz_embed_navigation_link_free(kzembed);
+
 	g_signal_emit_by_name(embed, "kz-net-start");
 
 	if (parent_class->net_start)
@@ -1006,9 +1021,6 @@ kz_moz_embed_net_stop (GtkMozEmbed *embed)
 	if (parent_class->net_stop)
 		parent_class->net_stop(embed);
 
-	/* First free previous link */
-	// kz_moz_embed_navigation_link_free(kzembed);
-
 	net_stop_proccess(kzembed);
 
 	g_signal_emit_by_name(embed, "kz-net-stop");
@@ -2305,7 +2317,8 @@ kz_moz_embed_can_go_nav_link (KzEmbed *kzembed,
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);
 
 	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
-	return (priv->nav_links[link]) ? TRUE : FALSE;
+	KzNavi *navi = KZ_NAVI(g_list_nth_data(priv->nav_links[link], 0));
+	return (navi && navi->uri) ? TRUE : FALSE;
 }
 
 static void
@@ -2315,42 +2328,80 @@ kz_moz_embed_go_nav_link (KzEmbed *kzembed,
 	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
 
 	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
-	g_return_if_fail(priv->nav_links[link]);
-	kz_moz_embed_load_url(kzembed, priv->nav_links[link]);
+	KzNavi *navi = KZ_NAVI(g_list_nth_data(priv->nav_links[link], 0));
+	g_return_if_fail(navi);
+
+	kz_moz_embed_load_url(kzembed, navi->uri);
+}
+
+
+static void
+kz_moz_embed_append_nav_link (KzEmbed *kzembed,
+			      KzEmbedNavLink link,
+			      KzNavi *navi)
+{
+	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
+	g_return_if_fail(link < KZ_EMBED_LINK_GUARD);
+	g_return_if_fail(navi);
+
+	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
+
+	priv->nav_links[link] = g_list_append(priv->nav_links[link],
+					      g_object_ref(navi));
 }
 
 static void
-kz_moz_embed_set_nav_link (KzEmbed *kzembed,
-			   const gchar *name,
-			   const gchar *link)
+kz_moz_embed_set_nth_nav_link (KzEmbed *kzembed,
+			       KzEmbedNavLink link,
+			       KzNavi *navi,
+			       guint n)
 {
 	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
-	g_return_if_fail(name);
-	g_return_if_fail(link);
+	g_return_if_fail(link < KZ_EMBED_LINK_GUARD);
+	g_return_if_fail(navi);
 
 	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
-	
-	for (int j = 0; j < N_LINKS; j++)
+	guint len = g_list_length(priv->nav_links[link]);
+	GList *nth_nav_link;
+
+	if (n >= len)
 	{
-		if (!strcasecmp(name, nav_names[j]))
+		int i;
+		for (i = len; i < n; i++)
 		{
-			if (priv->nav_links[j])
-				g_free(priv->nav_links[j]);
-			priv->nav_links[j] = g_strdup(link);
-			break;
+			priv->nav_links[link] =
+				g_list_append(priv->nav_links[link], NULL);
 		}
 	}
+	
+	nth_nav_link = g_list_nth(priv->nav_links[link], n);
+	if (nth_nav_link->data)
+		g_object_unref(nth_nav_link->data);
+	nth_nav_link->data = g_object_ref(navi);
 }
 
 
-static const gchar *
-kz_moz_embed_get_nav_link (KzEmbed *kzembed,
-			   KzEmbedNavLink link)
+static KzNavi *
+kz_moz_embed_get_nth_nav_link (KzEmbed *kzembed,
+			       KzEmbedNavLink link,
+			       guint n)
 {
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
 
-	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE (kzembed);
-	
+	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE(kzembed);
+
+	return KZ_NAVI(g_list_nth_data(priv->nav_links[link], n));
+}
+
+
+static GList *
+kz_moz_embed_get_nav_links (KzEmbed *kzembed,
+			    KzEmbedNavLink link)
+{
+	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
+
+	KzMozEmbedPrivate *priv = KZ_MOZ_EMBED_GET_PRIVATE(kzembed);
+
 	return priv->nav_links[link];
 }
 
