@@ -27,6 +27,9 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIComponentManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsCOMPtr.h>
 #define MOZILLA_STRICT_API
 #include <nsEmbedString.h>
@@ -35,7 +38,6 @@
 #include <nsIURL.h>
 #include <nsIMIMEInfo.h>
 #include <nsMemory.h>
-#include <nsComponentManagerUtils.h>
 
 #include "kz-mozlauncher.h"
 #include "kz-mozfilepicker.h"
@@ -63,10 +65,10 @@ KzContentHandler::~KzContentHandler()
 
 NS_IMETHODIMP KzContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 				     nsISupports *aContext,
-#ifdef MOZ_NSIHELPERAPPLAUNCHERDIALOG_NSPRBOOL_
-				     PRBool aForced)
-#else
+#ifdef HAVE_GECKO_1_8
 				     PRUint32 aForced)
+#else
+				     PRBool aForced)
 #endif
 {
 	nsresult rv;
