@@ -29,6 +29,9 @@
 #include <gtkmozembed_internal.h>
 
 #include <stdlib.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsILocalFile.h>
 #include <nsIDocShell.h>
