@@ -37,7 +37,6 @@ G_BEGIN_DECLS
 
 typedef struct _KzIO KzIO;
 typedef struct _KzIOClass KzIOClass;
-typedef struct _KzIOPrivate KzIOPrivate;
 
 typedef enum {
 	KZ_IO_READ,
@@ -53,8 +52,6 @@ struct _KzIO
 {
 	GObject parent;
 
-	KzIOPrivate *priv;
-
 	GIOChannel *iochannel;
 };
 
