@@ -515,7 +515,7 @@ prefs_font_create (void)
 	KzPrefsFont *prefsui = g_new0(KzPrefsFont, 1);
 	GtkWidget *main_vbox, *vbox, *hbox;
 	GtkWidget *frame, *table, *label, *option_menu, *combo, *spin;
-	gchar *str;
+	gchar *str = NULL;
 
 	prefsui->prefs_list = NULL;
 	prefsui->freeze     = FALSE;
@@ -757,13 +757,19 @@ prefs_font_create (void)
 			    	    str,
 			    	    font_styles, n_font_styles, style);
 	if (str)
+	{
 		g_free(str);
+		str =NULL;
+	}
 	mozilla_prefs_get_string("font.language.group", &str);
 	GTKUTIL_SET_DEFAULT_HISTORY(prefsui->lang_option_menu,
 			    	    str,
 			    	    fonts_language, n_fonts_languages, code);
 	if (str)
+	{
 		g_free(str);
+		str =NULL;
+	}
 	font_prefs_reset(prefsui);
 
 	return main_vbox;
