@@ -21,11 +21,19 @@
 #ifndef __PREFS_LANG_H__
 #define __PREFS_LANG_H__
 
-#include "kz-prefswin.h"
+#include "kz-prefs-win.h"
+#include "kz-langinfo.h"
+
+extern const LanguageInfo languages[];
+extern const guint n_languages;
+extern const AcceptLanguageInfo accept_languages[];
+extern const guint n_accept_languages;
+extern const EncodingAutodetectorInfo encoding_autodetectors[];
+extern const guint n_encoding_autodetectors;
 
 G_BEGIN_DECLS
 
-KzPrefsWinPageEntry *prefs_lang_get_entry (void);
+KzPrefsWinPageEntry *prefs_lang_get_entry (gint idx);
 
 G_END_DECLS
 
