@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-sidebar.c,v 1.7 2005/02/13 13:20:37 ikezoe Exp $
+ *  $Id: kz-bookmarks-sidebar.c,v 1.8 2005/08/03 22:40:36 ikezoe Exp $
  */
 
 #include "kz-bookmarks-sidebar.h"
@@ -227,7 +227,7 @@ cb_button_press (GtkWidget *widget, GdkEventButton *event,
 
 		uri = kz_bookmark_get_link(bookmark);
 
-		if (event->type == GDK_2BUTTON_PRESS)
+		if (event->button == 1)
 		{
 			if (kz_bookmark_is_folder(bookmark))
 			{
