@@ -19,9 +19,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <glib.h>
+#if (GLIB_MAJOR_VERSION == 2) && ((GLIB_MINOR_VERSION < 5) || \
+		((GLIB_MINOR_VERSION >= 5) && (GLIB_MICRO_VERSION < 4)))
 #include "goption.h"
 
-#include <glib.h>
 #include <glib/gi18n.h>
 
 #include <string.h>
@@ -1208,4 +1210,4 @@ g_option_group_set_translation_domain (GOptionGroup *group,
 				     g_strdup (domain),
 				     g_free);
 } 
-
+#endif
